@@ -1,5 +1,5 @@
 #
-# $Id: Headers.pm,v 1.19 1996/05/26 10:38:55 aas Exp $
+# $Id: Headers.pm,v 1.30 1997/12/10 13:08:30 aas Exp $
 
 package HTTP::Headers;
 
@@ -10,19 +10,21 @@ HTTP::Headers - Class encapsulating HTTP Message headers
 =head1 SYNOPSIS
 
  require HTTP::Headers;
- $request = new HTTP::Headers;
+ $h = new HTTP::Headers;
 
 =head1 DESCRIPTION
 
-The C<HTTP::Headers> class encapsulate HTTP style message headers.
-The headers consist of attribute value pairs which may be repeated,
+The C<HTTP::Headers> class encapsulates HTTP-style message headers.
+The headers consist of attribute-value pairs, which may be repeated,
 and which are printed in a particular order.
 
 Instances of this class are usually created as member variables of the
-C<HTTP::Request> and C<HTTP::Response> classes, internally to the
+C<HTTP::Request> and C<HTTP::Response> classes, internal to the
 library.
 
-=head1 METHODS
+The following methods are available:
+
+=over 4
 
 =cut
 
@@ -39,21 +41,22 @@ require Carp;
 #    - Request-Headers
 #    - Response-Headers
 #    - Entity-Headers
-# (From draft-ietf-http-v11-spec-00.ps, Nov 22, 1995)
+# (From draft-ietf-http-v11-spec-rev-01, Nov 21, 1997)
 
 my @header_order = qw(
-   Cache-Control Connection Date Forwarded Message-ID MIME-Version
-   Pragma Upgrade
+   Cache-Control Connection Date Pragma Transfer-Encoding Upgrade Trailer Via
 
-   Accept Accept-Charset Accept-Encoding Accept-Language Authorization
-   From Host If-Modified-Since Proxy-Authorization Orig-URI
-   Range Referer Unless User-Agent
+   Accept Accept-Charset Accept-Encoding Accept-Language
+   Authorization Expect From Host
+   If-Modified-Since If-Match If-None-Match If-Range If-Unmodified-Since
+   Max-Forwards Proxy-Authorization Range Referer TE User-Agent
 
-   Location Proxy-Authenticate Public Retry-After Server WWW-Authenticate
+   Accept-Ranges Age Location Proxy-Authenticate Retry-After Server Vary
+   Warning WWW-Authenticate
 
-   Allow Content-Encoding Content-Language Content-Length
-   Content-MD5 Content-Range Content-Type
-   Expires Last-Modified Link Title Transfer-Encoding URI Version
+   Allow Content-Base Content-Encoding Content-Language Content-Length
+   Content-Location Content-MD5 Content-Range Content-Type
+   ETag Expires Last-Modified
 );
 
 # Make alternative representations of @header_order.  This is used
@@ -69,15 +72,15 @@ for (@header_order) {
 
 
 
-=head2 $h = new HTTP::Headers
+=item $h = new HTTP::Headers
 
 Constructs a new C<HTTP::Headers> object.  You might pass some initial
-attribute value pairs as parameters to the constructor.  E.g.:
+attribute-value pairs as parameters to the constructor.  I<E.g.>:
 
  $h = new HTTP::Headers
-     'Content-Type' => 'text/html',
-     'MIME-Version' => '1.0',
-     'Date'         => 'Thu, 03 Feb 1994 00:00:00 GMT';
+     Date         => 'Thu, 03 Feb 1994 00:00:00 GMT',
+     Content_Type => 'text/html; version=3.2',
+     Content_Base => 'http://www.sn.no/';
 
 =cut
 
@@ -93,21 +96,26 @@ sub new
 }
 
 
-=head2 $h->header($field [=> $val],...)
+=item $h->header($field [=> $val],...)
 
-Get/Set the value of a request header.  The header field name is not
-case sensitive.  The value argument may be a scalar or a reference to
-a list of scalars. If the value argument is not defined then the
-header is not modified.
+Get or set the value of a header.  The header field name is not case
+sensitive.  To make the life easier for perl users who wants to avoid
+quoting before the => operator, you can use '_' as a synonym for '-'
+in header names.
+
+The value argument may be a scalar or a reference to a list of
+scalars. If the value argument is not defined, then the header is not
+modified.
 
 The header() method accepts multiple ($field => $value) pairs.
 
 The list of previous values for the last $field is returned.  Only the
 first header value is returned in scalar context.
 
- $header->header('MIME-Version' => '1.0',
-		 'User-Agent'   => 'My-Web-Client/0.01');
- $header->header('Accept' => "text/html, text/plain, image/*");
+ $header->header(MIME_Version => '1.0',
+		 User_Agent   => 'My-Web-Client/0.01');
+ $header->header(Accept => "text/html, text/plain, image/*");
+ $header->header(Accept => [qw(text/html text/plain image/*)]);
  @accepts = $header->header('Accept');
 
 =cut
@@ -125,6 +133,7 @@ sub header
 sub _header
 {
     my($self, $field, $val, $push) = @_;
+    $field =~ tr/_/-/;  # allow use of '_' as alternative to '-' in fields
 
     # $push is only used interally sub push_header
 
@@ -174,7 +183,7 @@ sub _header_cmp
 }
 
 
-=head2 $h->scan(\&doit)
+=item $h->scan(\&doit)
 
 Apply a subroutine to each header in turn.  The callback routine is
 called with two parameters; the name of the field and a single value.
@@ -201,16 +210,17 @@ sub scan
 }
 
 
-=head2 $h->as_string([$endl])
+=item $h->as_string([$endl])
 
 Return the header fields as a formatted MIME header.  Since it
 internally uses the C<scan()> method to build the string, the result
 will use case as suggested by HTTP Spec, and it will follow
 recommended "Good Practice" of ordering the header fieds.  Long header
-values are I<never> folded.
+values are not folded. 
 
 The optional parameter specifies the line ending sequence to use.  The
-default is C<"\n">.
+default is C<"\n">.  Embedded "\n" characters in the header will be
+substitued with this line ending sequence.
 
 =cut
 
@@ -222,8 +232,13 @@ sub as_string
     my @result = ();
     $self->scan(sub {
 	my($field, $val) = @_;
-	Carp::croak("HTTP header ($field) contains newline")
-	   if $val =~ /\n/;
+	if ($val =~ /\n/) {
+	    # must handle header values with embedded newlines with care
+	    $val =~ s/\s+$//;          # trailing newlines and space must go
+	    $val =~ s/\n\n+/\n/g;      # no empty lines
+	    $val =~ s/\n([^\040\t])/\n $1/g;  # intial space for continuation
+	    $val =~ s/\n/$endl/g;      # substitute with requested line ending
+	}
 	push(@result, "$field: $val");
     });
 
@@ -237,23 +252,25 @@ sub as_string
 # autoloaded section of the code, so we keep the documentation before
 # the __DATA__ token.
 
-=head2 $h->push_header($field, $val)
+=item $h->push_header($field, $val)
 
 Add a new field value of the specified header.  The header field name
 is not case sensitive.  The field need not already have a
 value. Previous values for the same field are retained.  The argument
 may be a scalar or a reference to a list of scalars.
 
- $header->push_header('Accept' => 'image/jpeg');
+ $header->push_header(Accept => 'image/jpeg');
 
-=head2 $h->remove_header($field,...)
+=item $h->remove_header($field,...)
 
 This function removes the headers with the specified names.
 
-=head2 $h->clone
+=item $h->clone
 
 Returns a copy of this HTTP::Headers object.
 
+=back
+
 =head1 CONVENIENCE METHODS
 
 The most frequently used headers can also be accessed through the
@@ -261,49 +278,53 @@ following convenience methods.  These methods can both be used to read
 and to set the value of a header.  The header value is set if you pass
 an argument to the method.  The old header value is always returned.
 
-Methods that deal with dates/time always convert their value to system
+Methods that deal with dates/times always convert their value to system
 time (seconds since Jan 1, 1970) and they also expect this kind of
 value when the header value is set.
 
-=head2 $h->date
+=over 4
+
+=item $h->date
 
 This header represents the date and time at which the message was
-originated. E.g.:
+originated. I<E.g.>:
 
   $h->date(time);  # set current date
 
-=head2 $h->expires
+=item $h->expires
 
 This header gives the date and time after which the entity should be
 considered stale.
 
-=head2 $h->if_modified_since
+=item $h->if_modified_since
+
+=item $h->if_unmodified_since
 
 This header is used to make a request conditional.  If the requested
 resource has not been modified since the time specified in this field,
-then the server til return a "304 Not Modified" response instead of
+then the server will return a C<"304 Not Modified"> response instead of
 the document itself.
 
-=head2 $h->last_modified
+=item $h->last_modified
 
 This header indicates the date and time at which the resource was last
-modified. E.g.:
+modified. I<E.g.>:
 
   # check if document is more than 1 hour old
   if ($h->last_modified < time - 60*60) {
 	...
   }
 
-=head2 $h->content_type
+=item $h->content_type
 
 The Content-Type header field indicates the media type of the message
-content. E.g.:
+content. I<E.g.>:
 
   $h->content_type('text/html');
 
 The value returned will be converted to lower case, and potential
-parameters will be chopped off and returned as a separate value if
-array context.  This makes it safe to do the following:
+parameters will be chopped off and returned as a separate value if in
+an array context.  This makes it safe to do the following:
 
   if ($h->content_type eq 'text/html') {
      # we enter this place even if the real header value happens to
@@ -311,34 +332,42 @@ array context.  This makes it safe to do the following:
      ...
   }
 
-=head2 $h->content_encoding
+=item $h->content_encoding
 
 The Content-Encoding header field is used as a modifier to the
 media type.  When present, its value indicates what additional
 encoding mechanism has been applied to the resource.
 
-=head2 $h->content_length
+=item $h->content_length
 
 A decimal number indicating the size in bytes of the message content.
 
-=head2 $h->title
+=item $h->content_language
 
-The title of the document.  Will be obtained from the
-<TITLE>...</TITLE> element of HTML documentents.
+The natural language(s) of the intended audience for the message
+content.  The value is one or more language tags as defined by RFC
+1766.  Eg. "no" for Norwegian and "en-US" for US-English.
 
-=head2 $h->user_agent
+=item $h->title
+
+The title of the document.  In libwww-perl this header will be
+initialized automatically from the E<lt>TITLE>...E<lt>/TITLE> element
+of HTML documents.  I<This header is no longer part of the HTTP
+standard.>
+
+=item $h->user_agent
 
 This header field is used in request messages and contains information
-about the user agent originating the request.  E.g.:
+about the user agent originating the request.  I<E.g.>:
 
   $h->user_agent('Mozilla/1.2');
 
-=head2 $h->server
+=item $h->server
 
 The server header field contains information about the software being
-used by the origin server program handling the request.
+used by the originating server program handling the request.
 
-=head2 $h->from
+=item $h->from
 
 This header should contain an Internet e-mail address for the human
 user who controls the requesting user agent.  The address should be
@@ -346,33 +375,56 @@ machine-usable, as defined by RFC822.  E.g.:
 
   $h->from('Gisle Aas <aas@sn.no>');
 
-=head2 $h->referer
+=item $h->referer
 
 Used to specify the address (URI) of the document from which the
 requested resouce address was obtained.
 
-=head2 $h->www_authenticate
+=item $h->www_authenticate
 
 This header must be included as part of a "401 Unauthorized" response.
 The field value consist of a challenge that indicates the
 authentication scheme and parameters applicable to the requested URI.
 
-=head2 $h->authorization
+=item $h->proxy_authenticate
 
-A user agent that wishes to authenticate itself with a server, may do
-so by including this header.
+This header must be included in a "407 Proxy Authentication Required"
+response.
 
-=head2 $h->authorization_basic
+=item $h->authorization
+
+=item $h->proxy_authorization
+
+A user agent that wishes to authenticate itself with a server or a
+proxy, may do so by including these headers.
+
+=item $h->authorization_basic
 
 This method is used to get or set an authorization header that use the
 "Basic Authentication Scheme".  In array context it will return two
 values; the user name and the password.  In scalar context it will
 return I<"uname:password"> as a single string value.
 
-When used to set the header value, it expects two arguments.  E.g.:
+When used to set the header value, it expects two arguments.  I<E.g.>:
 
   $h->authorization_basic($uname, $password);
 
+The method will croak if the $uname contains a colon ':'.
+
+=item $h->proxy_authorization_basic
+
+Same as authorization_basic() but will set the "Proxy-Authorization"
+header instead.
+
+=back
+
+=head1 COPYRIGHT
+
+Copyright 1995-1997 Gisle Aas.
+
+This library is free software; you can redistribute it and/or
+modify it under the same terms as Perl itself.
+
 =cut
 
 1;
@@ -396,9 +448,10 @@ sub push_header
 
 sub remove_header
 {
-    my $self = shift;
+    my($self, @fields) = @_;
     my $field;
-    foreach $field (@_) {
+    foreach $field (@fields) {
+	$field =~ tr/_/-/;
 	delete $self->{'_header'}{lc $field};
     }
 }
@@ -416,15 +469,21 @@ sub _date_header
     HTTP::Date::str2time($old);
 }
 
-sub date              { shift->_date_header('Date',              @_); }
-sub expires           { shift->_date_header('Expires',           @_); }
-sub if_modified_since { shift->_date_header('If-Modified-Since', @_); }
-sub last_modified     { shift->_date_header('Last-Modified',     @_); }
+sub date                { shift->_date_header('Date',                @_); }
+sub expires             { shift->_date_header('Expires',             @_); }
+sub if_modified_since   { shift->_date_header('If-Modified-Since',   @_); }
+sub if_unmodified_since { shift->_date_header('If-Unmodified-Since', @_); }
+sub last_modified       { shift->_date_header('Last-Modified',       @_); }
+
+# This is used as a private LWP extention.  The Client-Date header is
+# added as a timestamp to a response when it has been received.
+sub client_date         { shift->_date_header('Client-Date',         @_); }
 
 # The retry_after field is dual format (can also be a expressed as
 # number of seconds from now), so we don't provide an easy way to
 # access it until we have know how both these interfaces can be
-# addressed.
+# addressed.  One possibility is to return a negative value for
+# relative seconds and a positive value for epoch based time values.
 #sub retry_after       { shift->_date_header('Retry-After',       @_); }
 
 sub content_type      {
@@ -436,6 +495,7 @@ sub content_type      {
 
 sub title             { (shift->_header('Title',            @_))[0] }
 sub content_encoding  { (shift->_header('Content-Encoding', @_))[0] }
+sub content_language  { (shift->_header('Content-Language', @_))[0] }
 sub content_length    { (shift->_header('Content-Length',   @_))[0] }
 
 sub user_agent        { (shift->_header('User-Agent',       @_))[0] }
@@ -443,18 +503,28 @@ sub server            { (shift->_header('Server',           @_))[0] }
 
 sub from              { (shift->_header('From',             @_))[0] }
 sub referer           { (shift->_header('Referer',          @_))[0] }
+sub etag              { (shift->_header('ETag',             @_))[0] }
+sub warning           { (shift->_header('Warning',          @_))[0] }
 
 sub www_authenticate  { (shift->_header('WWW-Authenticate', @_))[0] }
 sub authorization     { (shift->_header('Authorization',    @_))[0] }
 
-sub authorization_basic {
+sub proxy_authenticate  { (shift->_header('Proxy-Authenticate',  @_))[0] }
+sub proxy_authorization { (shift->_header('Proxy-Authorization', @_))[0] }
+
+sub authorization_basic       { shift->_basic_auth("Authorization",       @_) }
+sub proxy_authorization_basic { shift->_basic_auth("Proxy-Authorization", @_) }
+
+sub _basic_auth {
     require MIME::Base64;
-    my($self, $user, $passwd) = @_;
-    my($old) = $self->_header('Authorization');
+    my($self, $h, $user, $passwd) = @_;
+    my($old) = $self->_header($h);
     if (defined $user) {
+	Carp::croak("Basic authorization user name can't contain ':'")
+	  if $user =~ /:/;
 	$passwd = '' unless defined $passwd;
-	$self->_header('Authorization',
-		       'Basic ' . MIME::Base64::encode("$user:$passwd", ''));
+	$self->_header($h => 'Basic ' .
+                             MIME::Base64::encode("$user:$passwd", ''));
     }
     if (defined $old && $old =~ s/^\s*Basic\s+//) {
 	my $val = MIME::Base64::decode($old);
