@@ -1,4 +1,4 @@
-# $Id: Protocol.pm,v 1.22 1996/05/26 10:41:39 aas Exp $
+# $Id: Protocol.pm,v 1.30 1997/12/15 20:29:10 aas Exp $
 
 package LWP::Protocol;
 
@@ -6,6 +6,12 @@ package LWP::Protocol;
 
 LWP::Protocol - Base class for LWP protocols
 
+=head1 SYNOPSIS
+
+ package LWP::Protocol::foo;
+ require LWP::Protocol;
+ @ISA=qw(LWP::Protocol);
+
 =head1 DESCRIPTION
 
 This class is used a the base class for all protocol implementations
@@ -22,12 +28,9 @@ method which is used to service a request. The overridden method can
 make use of the collect() function to collect together chunks of data
 as it is received.
 
-=head1 SEE ALSO
+The following methods and functions are provided:
 
-Inspect the F<LWP/Protocol/file.pm> and F<LWP/Protocol/http.pm> files
-for examples of usage.
-
-=head1 METHODS AND FUNCTIONS
+=over 4
 
 =cut
 
@@ -44,7 +47,7 @@ require HTML::HeadParser;
 my %ImplementedBy = (); # scheme => classname
 
 
-=head2 $prot = new HTTP::Protocol;
+=item $prot = new HTTP::Protocol;
 
 The LWP::Protocol constructor is inherited by subclasses. As this is a
 virtual base class this method should B<not> be called directly.
@@ -57,13 +60,13 @@ sub new
 
     my $self = bless {
 	'timeout' => 0,
-	'use_alarm' => 1,
+	'parse_head' => 1,
     }, $class;
     $self;
 }
 
 
-=head2 $prot = LWP::Protocol::create($url)
+=item $prot = LWP::Protocol::create($url)
 
 Create an object of the class implementing the protocol to handle the
 given scheme. This is a function, not a method. It is more an object
@@ -79,12 +82,11 @@ sub create
 	Carp::croak("Protocol scheme '$scheme' is not supported");
 
     # hand-off to scheme specific implementation sub-class
-    my $prot = new $impclass, $scheme;
-    return $prot;
+    return $impclass->new($scheme);
 }
 
 
-=head2 $class = LWP::Protocol::implementor($scheme, [$class])
+=item $class = LWP::Protocol::implementor($scheme, [$class])
 
 Get and/or set implementor class for a scheme.  Returns '' if the
 specified scheme is not supported.
@@ -101,15 +103,18 @@ sub implementor
     my $ic = $ImplementedBy{$scheme};
     return $ic if $ic;
 
+    return '' unless $scheme =~ /^([.+\-\w]+)$/;  # check valid URL schemes
+    $scheme = $1; # untaint
+    $scheme =~ s/[.+\-]/_/g;  # make it a legal module name
+
     # scheme not yet known, look for a 'use'd implementation
     $ic = "LWP::Protocol::$scheme";  # default location
     $ic = "LWP::Protocol::nntp" if $scheme eq 'news'; #XXX ugly hack
-    no strict qw(refs);
+    no strict 'refs';
     # check we actually have one for the scheme:
     unless (defined @{"${ic}::ISA"}) {
-	my $package = "$ic.pm";
-	$package =~ s|::|/|g;
-	eval { require "$package" };
+	# try to autoload it
+	eval "require $ic";
 	if ($@) {
 	    if ($@ =~ /^Can't locate/) { #' #emacs get confused by '
 		$ic = '';
@@ -123,7 +128,7 @@ sub implementor
 }
 
 
-=head2 $prot->request(...)
+=item $prot->request(...)
 
  $response = $protocol->request($request, $proxy, undef);
  $response = $protocol->request($request, $proxy, '/tmp/sss');
@@ -142,23 +147,24 @@ sub request
 }
 
 
-=head2 $prot->timeout($seconds)
+=item $prot->timeout($seconds)
 
 Get and set the timeout value in seconds
 
 
-=head2 $prot->use_alarm($yesno)
+=item $prot->parse_head($yesno)
 
-Indicates if the library is allowed to use the core alarm()
-function to implement timeouts.
+Should we initialize response headers from the <head> section of HTML
+documents.
 
 =cut
 
-sub timeout  { shift->_elem('timeout',  @_); }
-sub use_alarm { shift->_elem('use_alarm', @_); }
+sub timeout    { shift->_elem('timeout',    @_); }
+sub parse_head { shift->_elem('parse_head', @_); }
+sub max_size   { shift->_elem('max_size',   @_); }
 
 
-=head2 $prot->collect($arg, $response, $collector)
+=item $prot->collect($arg, $response, $collector)
 
 Called to collect the content of a request, and process it
 appropriately into a scalar, file, or by calling a callback.  If $arg
@@ -185,23 +191,30 @@ sub collect
 {
     my ($self, $arg, $response, $collector) = @_;
     my $content;
-    my($use_alarm, $timeout) = @{$self}{'use_alarm', 'timeout'};
+    my($parse_head, $timeout, $max_size) =
+      @{$self}{qw(parse_head timeout max_size)};
 
     my $parser;
-    if ($response->content_type eq 'text/html') {
+    if ($parse_head && $response->content_type eq 'text/html') {
 	$parser = HTML::HeadParser->new($response->{'_headers'});
     }
-    
+    my $content_size = 0;
+
     if (!defined($arg) || !$response->is_success) {
 	# scalar
 	while ($content = &$collector, length $$content) {
 	    if ($parser) {
 		$parser->parse($$content) or undef($parser);
 	    }
-	    alarm(0) if $use_alarm;
 	    LWP::Debug::debug("read " . length($$content) . " bytes");
 	    $response->add_content($$content);
-	    alarm($timeout) if $use_alarm;
+	    $content_size += length($$content);
+	    if ($max_size && $content_size > $max_size) {
+		LWP::Debug::debug("Aborting because size limit exceeded");
+		my $tot = $response->header("Content-Length") || 0;
+		$response->header("X-Content-Range", "bytes 0-$content_size/$tot");
+		last;
+	    }
 	}
     }
     elsif (!ref($arg)) {
@@ -209,15 +222,21 @@ sub collect
 	open(OUT, ">$arg") or
 	    return new HTTP::Response RC_INTERNAL_SERVER_ERROR,
 			  "Cannot write to '$arg': $!";
+        binmode(OUT);
         local($\) = ""; # ensure standard $OUTPUT_RECORD_SEPARATOR
 	while ($content = &$collector, length $$content) {
 	    if ($parser) {
 		$parser->parse($$content) or undef($parser);
 	    }
-	    alarm(0) if $use_alarm;
 	    LWP::Debug::debug("read " . length($$content) . " bytes");
 	    print OUT $$content;
-	    alarm($timeout) if $use_alarm;
+	    $content_size += length($$content);
+	    if ($max_size && $content_size > $max_size) {
+		LWP::Debug::debug("Aborting because size limit exceeded");
+		my $tot = $response->header("Content-Length") || 0;
+		$response->header("X-Content-Range", "bytes 0-$content_size/$tot");
+		last;
+	    }
 	}
 	close(OUT);
     }
@@ -227,7 +246,6 @@ sub collect
 	    if ($parser) {
 		$parser->parse($$content) or undef($parser);
 	    }
-	    alarm(0) if $use_alarm;
 	    LWP::Debug::debug("read " . length($$content) . " bytes");
             eval {
 		&$arg($$content, $response, $self);
@@ -237,7 +255,6 @@ sub collect
 		$response->header('X-Died' => $@);
 		last;
 	    }
-	    alarm($timeout) if $use_alarm
 	}
     }
     else {
@@ -248,7 +265,7 @@ sub collect
 }
 
 
-=head2 $prot->collect_once($arg, $response, $content)
+=item $prot->collect_once($arg, $response, $content)
 
 Can be called when the whole response content is available as
 $content.  This will invoke collect() with a collector callback that
@@ -269,3 +286,17 @@ sub collect_once
 }
 
 1;
+
+=head1 SEE ALSO
+
+Inspect the F<LWP/Protocol/file.pm> and F<LWP/Protocol/http.pm> files
+for examples of usage.
+
+=head1 COPYRIGHT
+
+Copyright 1995-1997 Gisle Aas.
+
+This library is free software; you can redistribute it and/or
+modify it under the same terms as Perl itself.
+
+=cut
